#include <algorithm>
#include <iostream>
#include <memory>

class Element {
	std::unique_ptr<Element> next;
	Element* prev;
	int value;

public:
	Element(std::unique_ptr<Element> n, Element* p, int v)
	    : next(std::move(n))
	    , prev(p)
	    , value(v) {}

	void setNext(std::unique_ptr<Element> n) {
		next = std::move(n);
	}

	const Element* getNext() const { return next.get(); }
	const Element* getPrev() const { return prev; }
	int getValue() const { return value; }

	~Element() {
		// just to show when elements get deallocated
		std::cout << "destroyed: " << value << "\n";
	}
};

class List {
	std::unique_ptr<Element> first;
	Element* last = nullptr;

public:
	void addToEnd(int v) {
		auto newLast = std::make_unique<Element>(nullptr, last, v);
		if (last) {
			Element* tmp = newLast.get();
			last->setNext(std::move(newLast));
			last = tmp;
		} else {
			first = std::move(newLast);
			last = first.get();
		}
	}

	void printAll() const {
		const Element* act = first.get();
		while (act) {
			std::cout << act->getValue() << ' ';
			act = act->getNext();
		}
		std::cout << '\n';
	}

	~List() {
		std::cout << "List destroyed\n";
	}
};

int main() {
	List list;
	for (int i : { 1, 2, 3, 4, 5 }) {
		list.addToEnd(i);
	}
	list.printAll();
}
